@@ -2,16 +2,16 @@
  * Copyright (C) 2000  Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
+ * modify it under the terms of the GNU General Public
  * License as published by the Free Software Foundation; either
  * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
+ * You should have received a copy of the GNU General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -388,6 +388,7 @@ static void
 bool_changed (GObject *object, GParamSpec *pspec, gpointer data)
 {
   GtkToggleButton *tb = GTK_TOGGLE_BUTTON (data);
+  GtkWidget *child;
   GValue val = { 0, };  
   
   g_value_init (&val, G_TYPE_BOOLEAN);
@@ -400,8 +401,9 @@ bool_changed (GObject *object, GParamSpec *pspec, gpointer data)
       unblock_controller (G_OBJECT (tb));
     }
 
-  gtk_label_set_text (GTK_LABEL (GTK_BIN (tb)->child), g_value_get_boolean (&val) ?
-                      "TRUE" : "FALSE");
+  child = gtk_bin_get_child (GTK_BIN (tb));
+  gtk_label_set_text (GTK_LABEL (child),
+                      g_value_get_boolean (&val) ? "TRUE" : "FALSE");
   
   g_value_unset (&val);
 }
@@ -909,16 +911,25 @@ property_widget (GObject    *object,
       {
 	GEnumClass *eclass;
 	gint j;
-	
+
+#if GTK_CHECK_VERSION (2, 91, 2)
+	prop_edit = gtk_combo_box_text_new ();
+#else
 	prop_edit = gtk_combo_box_new_text ();
+#endif
 	
 	eclass = G_ENUM_CLASS (g_type_class_ref (spec->value_type));
 	
 	j = 0;
 	while (j < eclass->n_values)
 	  {
+#if GTK_CHECK_VERSION (2, 91, 2)
+	    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (prop_edit),
+	                                    eclass->values[j].value_name);
+#else
 	    gtk_combo_box_append_text (GTK_COMBO_BOX (prop_edit),
-	                               eclass->values[j].value_name);
+                                      eclass->values[j].value_name);
+#endif
 	    ++j;
 	  }
 	
