@@ -1,10 +1,10 @@
 ; AisleRiot - streets_and_alleys.scm
 ; Copyright (C) 2001 Rosanna Yuen <zana@webwynk.net>
 ;
-; This game is free software; you can redistribute it and/or modify
+; This program is free software: you can redistribute it and/or modify
 ; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2, or (at your option)
-; any later version.
+; the Free Software Foundation, either version 3 of the License, or
+; (at your option) any later version.
 ;
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,9 +12,7 @@
 ; GNU General Public License for more details.
 ;
 ; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
-; USA
+; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 (define (new-game)
   (initialize-playing-area)
@@ -155,7 +153,7 @@
 	 #f)
 	((empty-slot? slot2)
 	 (and (= (get-value (get-top-card slot1)) ace)
-	      (list 2 (get-name (get-top-card slot1)) (_"an empty Foundation"))))
+	      (list 2 (get-name (get-top-card slot1)) (_"an empty foundation"))))
 	((and (= (get-suit (get-top-card slot1))
 		 (get-suit (get-top-card slot2)))
 	      (= (get-value (get-top-card slot1))
