@@ -28,6 +28,7 @@
 
 #include <libgames-support/games-frame.h>
 #include <libgames-support/games-controls.h>
+#include <libgames-support/games-pause-action.h>
 
 #ifdef GGZ_CLIENT
 #include "ggz-network.h"
@@ -45,7 +46,6 @@ static GtkWidget *pref_dialog = NULL;
 static gint unpause = 0;
 extern GtkWidget *window;
 extern GnibblesProperties *properties;
-extern gint paused;
 
 GtkWidget *start_level_label, *start_level_spin_button;
 GtkWidget *num_human, *num_ai;
@@ -66,7 +66,7 @@ static void
 destroy_cb (GtkWidget * widget, gpointer data)
 {
   if (unpause) {
-    pause_game_cb (NULL, 0);
+    gtk_action_activate (pause_action);
     unpause = 0;
   }
   network_set_preferences ();
@@ -230,9 +230,9 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
     return;
   }
 
-  if (!paused) {
+  if (!games_pause_action_get_is_paused (GAMES_PAUSE_ACTION (pause_action))) {
     unpause = 1;
-    pause_game_cb (NULL, 0);
+    gtk_action_activate (pause_action);
   }
 
   if (game_running ())
@@ -249,7 +249,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
 
   notebook = gtk_notebook_new ();
   gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (pref_dialog)->vbox),
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (pref_dialog))),
                      notebook);
 
   label = gtk_label_new (_("Game"));
@@ -468,15 +468,15 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
     gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
     gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1, 0, 0, 0, 0);
 
-    omenu = gtk_combo_box_new_text ();
+    omenu = gtk_combo_box_text_new ();
     gtk_label_set_mnemonic_widget (GTK_LABEL (label2), omenu);
-    gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Red"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Green"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Blue"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Yellow"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Cyan"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Purple"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Gray"));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (omenu), _("Red"));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (omenu), _("Green"));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (omenu), _("Blue"));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (omenu), _("Yellow"));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (omenu), _("Cyan"));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (omenu), _("Purple"));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (omenu), _("Gray"));
     g_signal_connect (GTK_OBJECT (omenu), "changed",
                       G_CALLBACK (set_worm_color_cb),
                       GINT_TO_POINTER (i));
